/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.sample.common

import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.os.Build
import android.os.Bundle
import android.view.*
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.snackbar.BaseTransientBottomBar.*
import com.google.android.material.snackbar.Snackbar
import com.kakao.sdk.auth.*
import com.kakao.sdk.auth.model.OAuthToken
import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.common.model.*
import com.kakao.sdk.common.util.KakaoCustomTabsClient
import com.kakao.sdk.link.LinkClient
import com.kakao.sdk.link.WebSharerClient
import com.kakao.sdk.navi.NaviClient
import com.kakao.sdk.navi.model.CoordType
import com.kakao.sdk.navi.model.Location
import com.kakao.sdk.navi.model.NaviOptions
import com.kakao.sdk.story.StoryApiClient
import com.kakao.sdk.talk.TalkApiClient
import com.kakao.sdk.template.*
import com.kakao.sdk.user.UserApiClient
import com.kakao.sdk.user.model.User
import kotlinx.android.synthetic.main.fragment_open.*
import java.io.File
import java.io.FileOutputStream
import java.util.concurrent.CompletableFuture

class OpenFragment : Fragment() {

    private lateinit var viewAdapter: RecyclerView.Adapter<*>
    private lateinit var viewManager: RecyclerView.LayoutManager

    val TAG = "KakaoSDKSample"

    private fun showSnackbar(any: Any) {
        val snackbar = Snackbar.make(view!!, any.toString(), LENGTH_LONG)
        val layout = snackbar.view as Snackbar.SnackbarLayout
        layout.minimumHeight = 100
        snackbar.show()
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        val context = this.context!!
        Log.view = view

        viewManager = LinearLayoutManager(context)
        viewAdapter = ApiAdapter(
            listOf(
                ApiAdapter.Item.Header("Kakao Login"),
                ApiAdapter.Item.ApiItem("isTalkLoginAvailable()") {
                    if (LoginClient.instance.isTalkLoginAvailable(context)) {
                        Log.i(TAG, "간편로그인 가능")
                    } else {
                        Log.i(TAG, "카카오톡이 설치되지 않음")
                    }
                },
                ApiAdapter.Item.ApiItem("loginWithKakaoTalk()") {
                    LoginClient.instance.loginWithKakaoTalk(context) { token, error ->
                        if (error != null) {
                            Log.e(TAG, "로그인 실패", error)
                        } else {
                            Log.i(TAG, "로그인 성공 ${token!!.accessToken}")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("loginWithKakaoAccount()") {
                    LoginClient.instance.loginWithKakaoAccount(context) { token, error ->
                        if (error != null) {
                            Log.e(TAG, "로그인 실패", error)
                        } else {
                            Log.i(TAG, "로그인 성공 ${token!!.accessToken}")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("Combination Login") {
                    // 로그인 조합 예제

                    // 로그인 공통 callback 구성
                    val callback: (OAuthToken?, Throwable?) -> Unit = { token, error ->
                        if (error != null) {
                            // 로그인 실패
                            // 아래는 로그인 요청의 대표적인 에러케이스 처리 예제입니다. 서비스에 최적화된 형태로 간소화하세요.
                            when (error) {
                                is ClientError ->
                                    when (error.reason) {
                                        ClientErrorCause.Cancelled ->
                                            Log.d(TAG, "취소됨 (back button)", error)
                                        ClientErrorCause.NotSupported ->
                                            Log.e(TAG, "지원되지 않음 (카톡 미설치)", error)
                                        else ->
                                            Log.e(TAG, "기타 클라이언트 에러", error)
                                    }
                                is OAuthError ->
                                    when (error.reason) {
                                        AuthErrorCause.AccessDenied ->
                                            Log.d(TAG, "취소됨 (동의 취소)", error)
                                        AuthErrorCause.Misconfigured ->
                                            Log.e(TAG, "개발자사이트 앱 설정에 키해시를 등록하세요. 현재 값: ${KakaoSdk.applicationContextInfo.signingKeyHash}", error)
                                        else ->
                                            Log.e(TAG, "기타 인증 에러", error)
                                    }
                                else ->
                                    // 에러처리에 대한 개선사항이 필요하면 데브톡(https://devtalk.kakao.com)으로 문의해주세요.
                                    Log.e(TAG, "기타 에러 (네트워크 장애 등..)", error)
                            }
                        }
                        else {
                            Log.i(TAG, "로그인 성공 ${token!!.accessToken}")
                        }
                    }

                    // 카카오톡이 설치되어 있으면 간편로그인, 아니면 CustomTabs 브라우저 로그인
                    if (LoginClient.instance.isTalkLoginAvailable(context)) {
                        LoginClient.instance.loginWithKakaoTalk(context, callback = callback)
                    } else {
                        LoginClient.instance.loginWithKakaoAccount(context, callback = callback)
                    }
                },
                ApiAdapter.Item.Header("User API"),
                ApiAdapter.Item.ApiItem("me()") {

                    // 사용자 정보 조회 (기본)
                    UserApiClient.instance.me { user, error ->
                        if (error != null) {
                            Log.e(TAG, "사용자 정보 요청 실패", error)
                        } else {
                            Log.i(TAG, "사용자 정보 요청 성공" +
                                    "\n회원번호: ${user!!.id}" +
                                    "\n이메일: ${user!!.kakaoAccount?.email}" +
                                    "\n닉네임: ${user!!.kakaoAccount?.profile?.nickname}")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("me() - if requires email") {
                    // 이메일 필수 시나리오에서 동적동의 예제

                    // 기본 예제
                    //  - 단점: callback hell
                    //
                    // 만약 재시도 이후에도 계속 needsAgreement를 체크해서 로직상으로 획득을 보장하고 싶다면, 재귀적인 구조가 필요함
                    // (v1 샘플앱이 그런 구조인데 메소드를 따로 따야해서 코드스닙으로 제공하기는 난해함, 개발가이드에선 주석으로 해결했었음)
                    // 지금 예제에서는 정상적으로 이메일 동의에 성공한다고 가정하고 재시도 이후 콜백을 단순하게 처리함
                    //
                    // scope 을 다른 값을 넣는다면, 동적동의에 성공하지만 이메일 scope 은 추가가 안될텐데.. (오타라면 invalid_scope 에러 발생)
                    // 이 케이스에서 만약 재귀적으로 체크한다면 무한 뺑뺑이가 돌게됨. 오히려 안좋은가??
                    // 과거에 개인정보팀 요구로 선택항목에 대해 동적동의를 받을 때 체크박스를 보여주던 적이 있었는데, 체크 안하고 동의가 가능해서 동적동의가 성공해도 scope이 추가 안되는 증상도 있었음
                    UserApiClient.instance.me { user, error ->
                        if (error != null) {
                            Log.e(TAG, "사용자 정보 요청 실패", error)
                        }
                        else {
                            if (user!!.kakaoAccount?.email != null) {
                                Log.i(TAG, "이메일: ${user!!.kakaoAccount?.email}")

                            } else if (user!!.kakaoAccount?.emailNeedsAgreement == false) {
                                Log.e(TAG, "이메일 없음. 꼭 필요하다면 동의항목 설정에서 수집 기능을 활성화 해보세요.")

                            } else if (user!!.kakaoAccount?.emailNeedsAgreement == true) {
                                Log.d(TAG, "사용자에게 이메일 제공 동의를 받아야 합니다.")

                                // 사용자에게 이메일 제공 동의 요청
                                val scopes = listOf("account_email")
                                LoginClient.instance.loginWithNewScopes(context!!, scopes) { token, error ->
                                    if (error != null) {
                                        Log.e(TAG, "이메일 제공 동의 실패", error)
                                    } else {
                                        Log.d(TAG, "allowed scopes: ${token!!.scopes}")

                                        // 사용자 정보 재요청
                                        UserApiClient.instance.me { user, error ->
                                            if (error != null) {
                                                Log.e(TAG, "사용자 정보 요청 실패", error)
                                            }
                                            else {
                                                Log.i(TAG, "이메일: ${user!!.kakaoAccount?.email}")
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }


//                    // 필수항목 지정했을 때 SDK 내부에서 처리해주는 방식을 제공한다면..
//                    //  - 장점: 아주 심플하고 원하는 것만 동적동의 받는다는 원래 취지를 잘 표현해줌
//                    //  - 단점: REST API, JS SDK 와 사용성이 다름
//                    //  - 장점이자 단점: 재귀적으로 재시도하므로 기본예제 설명에서 언급한 무한 뺑뺑이 가능성 있음
//                    //  - TODO: scope을 enum이나 const String으로 제공해서 오타 방지 및 자동완성 편의 제공
//                    UserApiClient.instance.me(requiredScopes = listOf("account_email", "birthday")) { user, error ->
//                        if (error != null) {
//                            Log.e(TAG, "사용자 정보 요청 실패", error)
//                        } else if (user!!.kakaoAccount?.email != null) {
//                            Log.i(TAG, "이메일: ${user!!.kakaoAccount?.email}\n생일: ${user!!.kakaoAccount?.birthday}")
//                        } else {
//                            Log.i(TAG, "이메일 없음. 꼭 필요하다면 동의항목 설정에서 수집 기능을 활성화 해보세요.")
//                        }
//                    }
//
//
//                    // SDK가 동기 방식 API 메소드를 제공해준다면..
//                    //  - 장점: 우리 동의처리에 대한 로직이 심플해짐, Android 진영에선 동기 방식도 같이 제공하는게 일반적임
//                    //  - 단점: 쓰레드 관리 주의 요함.
//                    Thread {
//                        try {
//                            var user = UserApiClient.instance.me()
//
//                            if (user.kakaoAccount?.emailNeedsAgreement == true) {
//                                Log.d(TAG, "사용자에게 이메일 제공 동의를 받아야 합니다.")
//
//                                // 사용자에게 이메일 제공 동의 요청
//                                val newScopes = listOf("account_email")
//                                val token = LoginClient.instance.loginWithNewScopes(context, newScopes)
//                                Log.d(TAG, "allowed scopes: ${token.scopes}")
//
//                                // 사용자 정보 재요청
//                                user = UserApiClient.instance.me()
//                            }
//
//                            if (user.kakaoAccount?.email == null) {
//                                Log.i(TAG, "이메일 없음. 꼭 필요하다면 동의항목 설정에서 수집 기능을 활성화 해보세요.")
//                            } else {
//                                Log.i(TAG, "이메일: ${user.kakaoAccount?.email}")
//                            }
//                        } catch (e: Throwable) {
//                            Log.e(TAG, "사용자 정보 요청 실패", e)
//                        }
//                    }.start()
//
//
//                    // SDK에서 재시도 기능을 제공해준다면..
//                    //  - 쪼금 장점: 기본예제에서 맨 마지막 me 호출 부분이 간편해짐
//                    //  - 장점이자 단점: 재귀적으로 재시도하므로 기본예제 설명에서 언급한 무한 뺑뺑이 가능성 있음
//                    UserApiClient.instance.me { task, user, error ->
//                        if (error != null) {
//                            Log.e(TAG, "사용자 정보 요청 실패", error)
//                        }
//                        else {
//                            if (user!!.kakaoAccount?.email != null) {
//                                Log.i(TAG, "이메일: ${user!!.kakaoAccount?.email}")
//
//                            } else if (user!!.kakaoAccount?.emailNeedsAgreement == false) {
//                                Log.e(TAG, "이메일 없음. 꼭 필요하다면 동의항목 설정에서 수집 기능을 활성화 해보세요.")
//
//                            } else if (user!!.kakaoAccount?.emailNeedsAgreement == true) {
//                                Log.d(TAG, "사용자에게 이메일 제공 동의를 받아야 합니다.")
//
//                                // 사용자에게 이메일 제공 동의 요청
//                                val scopes = listOf("profile")
//                                LoginClient.instance.loginWithNewScopes(context!!, scopes) { token, error ->
//                                    if (error != null) {
//                                        Log.e(TAG, "이메일 제공 동의 실패", error)
//                                    } else {
//                                        Log.d(TAG, "allowed scopes: ${token!!.scopes}")
//
//                                        // 사용자 정보 재요청. (현재 콜백 재사용)
//                                        task.retry()
//                                    }
//                                }
//                            }
//                        }
//                    }
//
//
//                    // CompletableFuture 를 써봤는데 더 복잡하네요..
//                    // 쓰레드 신경써야 하는 단점도 가져오면서 코드는 더 길고..
//                    // 굳이 장점을 찾자면 마지막 이메일과 에러처리를 한곳에서 하는 의미밖에..
//                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
//                        Thread {
//                            var future = CompletableFuture<User>()
//
//                            UserApiClient.instance.me { user, error ->
//                                if (error != null) {
//                                    future.completeExceptionally(error)
//                                }
//                                else if (user!!.kakaoAccount?.emailNeedsAgreement == true) {
//                                    Log.d(TAG, "사용자에게 이메일 제공 동의를 받아야 합니다.")
//
//                                    // 이메일 동의 요청
//                                    val newScopes = listOf("account_email")
//                                    LoginClient.instance.loginWithNewScopes(context, newScopes) { token, error ->
//                                        if (error != null) {
//                                            future.completeExceptionally(error)
//                                        }
//                                        else {
//                                            Log.d(TAG, "allowed scopes: ${token!!.scopes}")
//
//                                            // 사용자 정보 재요청
//                                            UserApiClient.instance.me { user, error ->
//                                                if (error != null) {
//                                                    future.completeExceptionally(error)
//                                                } else {
//                                                    future.complete(user)
//                                                }
//                                            }
//                                        }
//                                    }
//                                }
//                                else {
//                                    future.complete(user)
//                                }
//                            }
//
//                            try {
//                                val user = future.get()
//
//                                if (user.kakaoAccount?.email == null) {
//                                    Log.i(TAG, "이메일 없음. 꼭 필요하다면 동의항목 설정에서 수집 기능을 활성화 해보세요.")
//                                } else {
//                                    Log.i(TAG, "이메일: ${user.kakaoAccount?.email}")
//                                }
//                            } catch (e: Throwable) {
//                                Log.e(TAG, "사용자 정보 요청 실패", e)
//                            }
//                        }.start()
//                    }

                },
                ApiAdapter.Item.ApiItem("accessTokenInfo()") {
                    UserApiClient.instance.accessTokenInfo { info, error ->
                        if (info != null) {
                            showSnackbar("id:${info.id}, expires_in:${info.expiresIn}, millis:${info.expiresInMillis}")
                        } else if (error != null) {
                            showSnackbar(error)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("accessTokenInfo() * 3") {
                    for (i in 0..3) {
                        UserApiClient.instance.accessTokenInfo { info, error ->
                            if (info != null) {
                                showSnackbar(info)
                            } else if (error != null) {
                                showSnackbar(error)
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("updateProfile()") {
                    val properties = mapOf("nickname" to "${System.currentTimeMillis()}")
                    UserApiClient.instance.updateProfile(properties) { error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar("부가정보 업데이트 완료")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("shippingAddresses()") {
                    UserApiClient.instance.shippingAddresses { userShippingAddresses, error ->
                        if (userShippingAddresses != null) {
                            showSnackbar(userShippingAddresses)
                        } else if (error != null) {
                            showSnackbar(error)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("serviceTerms()") {
                    UserApiClient.instance.serviceTerms { userServiceTerms, error ->
                        if (userServiceTerms != null) {
                            showSnackbar(userServiceTerms)
                        } else if (error != null) {
                            showSnackbar(error)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("logout()") {
                    UserApiClient.instance.logout { error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar("로그아웃 성공")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("unlink()") {
                    UserApiClient.instance.unlink { error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar("연결끊기 성공")
                        }
                    }
                },
                ApiAdapter.Item.Header("KakaoTalk API"),
                ApiAdapter.Item.ApiItem("profile()") {
                    TalkApiClient.instance.profile { profile, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar(profile!!)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendCustomMemo()") {
                    val templateId = templateIds["customMessage"] as Long

                    TalkApiClient.instance.sendCustomMemo(templateId) { error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar("나에게 보내기 성공")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendDefaultMemo()") {
                    val template = defaultFeed

                    TalkApiClient.instance.sendDefaultMemo(template) { error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar("나에게 보내기 성공")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendScrapMemo()") {
                    val url = "https://developers.kakao.com"

                    TalkApiClient.instance.sendScrapMemo(url) { error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar("나에게 보내기 성공")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("friends()") {
                    TalkApiClient.instance.friends { friends, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar(friends!!)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendCustomMessage()") {
                    TalkApiClient.instance.friends { friends, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (friends != null) {
                            if (friends.elements.isEmpty()) {
                                showSnackbar("메시지 보낼 친구가 없습니다.")
                            } else {
                                FriendsActivity.startForResult(
                                    context!!,
                                    friends.elements.map { PickerItem(it.uuid, it.profileNickname, it.profileThumbnailImage) }
                                ) { selectedItems ->
                                    if (selectedItems.isEmpty()) return@startForResult

                                    val templateId = templateIds["customMessage"] as Long

                                    TalkApiClient.instance.sendCustomMessage(selectedItems, templateId) { result, error ->
                                        if (error != null) {
                                            showSnackbar(error)
                                        } else {
                                            showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverUuids}")
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendDefaultMessage()") {
                    TalkApiClient.instance.friends { friends, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (friends != null) {
                            if (friends.elements.isEmpty()) {
                                showSnackbar("메시지 보낼 친구가 없습니다.")
                            } else {
                                FriendsActivity.startForResult(
                                    context!!,
                                    friends.elements.map { PickerItem(it.uuid, it.profileNickname, it.profileThumbnailImage) }
                                ) { selectedItems ->
                                    if (selectedItems.isEmpty()) return@startForResult

                                    val template = defaultFeed

                                    TalkApiClient.instance.sendDefaultMessage(selectedItems, template) { result, error ->
                                        if (error != null) {
                                            showSnackbar(error)
                                        } else {
                                            showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverUuids}")
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("sendScrapMessage()") {
                    TalkApiClient.instance.friends { friends, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (friends != null) {
                            if (friends.elements.isEmpty()) {
                                showSnackbar("메시지 보낼 친구가 없습니다.")
                            } else {
                                FriendsActivity.startForResult(
                                    context!!,
                                    friends.elements.map { PickerItem(it.uuid, it.profileNickname, it.profileThumbnailImage) }
                                ) { selectedItems ->
                                    if (selectedItems.isEmpty()) return@startForResult

                                    val url = "https://developers.kakao.com"

                                    TalkApiClient.instance.sendScrapMessage(selectedItems, url) { result, error ->
                                        if (error != null) {
                                            showSnackbar(error)
                                        } else {
                                            showSnackbar("메시지 보내기 성공 ${result!!.successfulReceiverUuids}")
                                        }
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("channels()") {
                    TalkApiClient.instance.channels { relations, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar(relations!!)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("addChannelUrl()") {
                    val url = TalkApiClient.instance.addChannelUrl("_ZeUTxl")

                    KakaoCustomTabsClient.openWithDefault(context!!, url)
                },
                ApiAdapter.Item.ApiItem("channelChatUrl()") {
                    val url = TalkApiClient.instance.channelChatUrl("_ZeUTxl")

                    KakaoCustomTabsClient.openWithDefault(context!!, url)
                },
                ApiAdapter.Item.Header("KakaoStory API"),
                ApiAdapter.Item.ApiItem("isStoryUser()") {
                    StoryApiClient.instance.isStoryUser { isStoryUser, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar("isStoryUser: $isStoryUser")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("profile()") {
                    StoryApiClient.instance.profile { profile, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar(profile!!)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("stories()") {
                    StoryApiClient.instance.stories { stories, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar(stories!!)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("story(id:) - first of stories") {
                    StoryApiClient.instance.stories { stories, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (stories != null) {
                            if (stories.isEmpty()) {
                                showSnackbar("내 스토리가 하나도 없어요 ㅠㅠ")
                            } else {
                                StoryApiClient.instance.story(stories.first().id) { story, error ->
                                    if (error != null) {
                                        showSnackbar(error)
                                    } else if (story != null) {
                                        showSnackbar(story)
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("delete(id:) - first of stories") {
                    StoryApiClient.instance.stories { stories, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else if (stories != null) {
                            if (stories.isEmpty()) {
                                showSnackbar("내 스토리가 하나도 없어요 ㅠㅠ")
                            } else {
                                val id = stories.first().id

                                StoryApiClient.instance.delete(id) { error ->
                                    if (error != null) {
                                        showSnackbar(error)
                                    } else {
                                        showSnackbar("스토리 삭제 완료 [$id]")
                                    }
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("linkInfo()") {
                    StoryApiClient.instance.linkInfo("https://www.kakaocorp.com") { linkInfo, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar(linkInfo!!)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("postNote()") {
                    StoryApiClient.instance.postNote("Posting note from Kakao SDK Sample.") { storyPostResult, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar(storyPostResult?.id!!)
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("postLink()") {
                    StoryApiClient.instance.linkInfo("https://www.kakaocorp.com") { linkInfo, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            StoryApiClient.instance.postLink(linkInfo!!, "Posting link from Kakao SDK Sample.") { storyPostResult, error ->
                                if (error != null) {
                                    showSnackbar(error)
                                } else {
                                    showSnackbar(storyPostResult?.id!!)
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("postPhoto()") {
                    val bitmap = BitmapFactory.decodeResource(resources, R.drawable.sample1)
                    val file = File(context!!.cacheDir, "sample1.png")
                    val stream = FileOutputStream(file)
                    bitmap.compress(Bitmap.CompressFormat.PNG, 100, stream)
                    stream.close()

                    StoryApiClient.instance.upload(listOf(file)) { uploadedPaths, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            StoryApiClient.instance.postPhoto(uploadedPaths!!, "Posting photo from Kakao SDK Sample.") { storyPostResult, error ->
                                if (error != null) {
                                    showSnackbar(error)
                                } else {
                                    showSnackbar(storyPostResult?.id!!)
                                }
                            }
                        }
                    }
                },
                ApiAdapter.Item.Header("KakaoLink API"),
                ApiAdapter.Item.ApiItem("isKakaoLinkAvailable()") {
                    val available = LinkClient.instance.isKakaoLinkAvailable(context!!)
                    showSnackbar("isKakaoLinkAvailable: $available")
                },
                ApiAdapter.Item.ApiItem("customTemplate()") {
                    val templateId = templateIds["customMemo"] as Long

                    LinkClient.instance.customTemplate(context!!, templateId) { linkResult, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            startActivity(linkResult!!.intent)

                            Log.w("LinkResult", "Warning Msg: ${linkResult!!.warningMsg}")
                            Log.w("LinkResult", "Argument Msg: ${linkResult!!.argumentMsg}")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("scrapTemplate()") {
                    val url = "https://developers.kakao.com"

                    LinkClient.instance.scrapTemplate(context!!, url) { linkResult, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            startActivity(linkResult!!.intent)

                            Log.w("LinkResult", "Warning Msg: ${linkResult!!.warningMsg}")
                            Log.w("LinkResult", "Argument Msg: ${linkResult!!.argumentMsg}")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("defaultTemplate()") {
                    LinkClient.instance.defaultTemplate(context!!, defaultFeed) { linkResult, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            startActivity(linkResult!!.intent)

                            Log.w("LinkResult", "Warning Msg: ${linkResult!!.warningMsg}")
                            Log.w("LinkResult", "Argument Msg: ${linkResult!!.argumentMsg}")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("defaultTemplate() - list") {
                    LinkClient.instance.defaultTemplate(context!!, defaultList) { linkResult, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            startActivity(linkResult!!.intent)

                            Log.w("LinkResult", "Warning Msg: ${linkResult!!.warningMsg}")
                            Log.w("LinkResult", "Argument Msg: ${linkResult!!.argumentMsg}")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("defaultTemplate() - location") {
                    LinkClient.instance.defaultTemplate(context!!, defaultLocation) { linkResult, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            startActivity(linkResult!!.intent)

                            Log.w("LinkResult", "Warning Msg: ${linkResult!!.warningMsg}")
                            Log.w("LinkResult", "Argument Msg: ${linkResult!!.argumentMsg}")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("defaultTemplate() - commerce") {
                    LinkClient.instance.defaultTemplate(context!!, defaultCommerce) { linkResult, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            startActivity(linkResult!!.intent)

                            Log.w("LinkResult", "Warning Msg: ${linkResult!!.warningMsg}")
                            Log.w("LinkResult", "Argument Msg: ${linkResult!!.argumentMsg}")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("defaultTemplate() - text") {
                    LinkClient.instance.defaultTemplate(context!!, defaultText) { linkResult, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            startActivity(linkResult!!.intent)

                            Log.w("LinkResult", "Warning Msg: ${linkResult!!.warningMsg}")
                            Log.w("LinkResult", "Argument Msg: ${linkResult!!.argumentMsg}")
                        }
                    }
                },
                ApiAdapter.Item.ApiItem("customTemplateUri() - web sharer") {
                    val shareUrl = WebSharerClient.instance.customTemplateUri(templateIds["customMemo"] as Long)

                    KakaoCustomTabsClient.openWithDefault(context!!, shareUrl)
                },
                ApiAdapter.Item.ApiItem("scrapTemplateUri() - web sharer") {
                    val shareUrl = WebSharerClient.instance.scrapTemplateUri("https://developers.kakao.com")

                    KakaoCustomTabsClient.openWithDefault(context!!, shareUrl)
                },
                ApiAdapter.Item.ApiItem("defaultTemplateUri() - web sharer") {
                    val shareUrl = WebSharerClient.instance.defaultTemplateUri(defaultFeed)

                    KakaoCustomTabsClient.openWithDefault(context!!, shareUrl)
                },
                ApiAdapter.Item.ApiItem("defaultTemplateUri() - location with web sharer") {
                    val shareUrl = WebSharerClient.instance.defaultTemplateUri(defaultLocation)

                    KakaoCustomTabsClient.openWithDefault(context!!, shareUrl)
                },
                ApiAdapter.Item.ApiItem("uploadImage()") {
                    val bitmap = BitmapFactory.decodeResource(resources, R.drawable.sample1)
                    val file = File(context!!.cacheDir, "sample1.png")
                    val stream = FileOutputStream(file)
                    bitmap.compress(Bitmap.CompressFormat.PNG, 100, stream)
                    stream.close()
                    LinkClient.instance.uploadImage(file) { imageUploadResult, error ->
                        if (error != null) {
                            showSnackbar(error)
                        } else {
                            showSnackbar(imageUploadResult!!)
                        }
                    }
                },
                ApiAdapter.Item.Header("KakaoNavi API"),
                ApiAdapter.Item.ApiItem("isKakaoNaviInstalled()") {
                    val naviInstalled = NaviClient.instance.isKakaoNaviInstalled(context!!)

                    showSnackbar("isKakaoNaviInstalled: $naviInstalled")
                },
                ApiAdapter.Item.ApiItem("shareDestinationIntent()") {
                    val intent =
                        NaviClient.instance.shareDestinationIntent(
                            Location("카카오 판교 오피스", 127.10821222694533, 37.40205604363057),
                            NaviOptions(coordType = CoordType.WGS84)
                        )
                    startActivity(intent)
                },
                ApiAdapter.Item.ApiItem("navigateIntent()") {
                    val intent =
                        NaviClient.instance.navigateIntent(
                            Location("카카오 판교 오피스", 127.10821222694533, 37.40205604363057),
                            NaviOptions(coordType = CoordType.WGS84)
                        )
                    startActivity(intent)
                },
                ApiAdapter.Item.ApiItem("shareDestinationUri()") {
                    val uri =
                        NaviClient.instance.navigateUrl(
                            Location("카카오 판교 오피스", 127.10821222694533, 37.40205604363057),
                            NaviOptions(coordType = CoordType.WGS84)
                        )
                    KakaoCustomTabsClient.openWithDefault(context!!, uri)
                }
            )
        )
        recyclerView.apply {
            setHasFixedSize(true)
            addItemDecoration(
                DividerItemDecoration(context, DividerItemDecoration.VERTICAL)
            )
            layoutManager = viewManager
            adapter = viewAdapter
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let {
            templateIds = it
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        val view = inflater.inflate(R.layout.fragment_open, container, false)
//        setHasOptionsMenu(true)
        return view
    }

//    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
//        inflater.inflate(R.menu.menu_main, menu)
//        super.onCreateOptionsMenu(menu, inflater)
//
//    }

    private lateinit var templateIds: Bundle

    companion object {
        fun newInstance(templateIds: Bundle): OpenFragment =
            OpenFragment().apply {
                arguments = templateIds
            }

        val defaultFeed = FeedTemplate(
            content = Content(
                "딸기 치즈 케익",
                "http://mud-kage.kakao.co.kr/dn/Q2iNx/btqgeRgV54P/VLdBs9cvyn8BJXB3o7N8UK/kakaolink40_original.png",
                Link(
                    "https://developers.kakao.com",
                    "https://developers.kakao.com"
                ),
                description = "#케익 #딸기 #삼평동 #카페 #분위기 #소개팅"
            ),
            social = Social(),
            buttons = listOf(
                Button(
                    "웹으로 보기",
                    Link(
                        "https://developers.kakao.com",
                        "https://developers.kakao.com"
                    )
                ),
                Button(
                    "앱으로 보기",
                    Link(
                        "https://developers.kakao.com",
                        "https://developers.kakao.com",
                        mapOf("key1" to "value1", "key2" to "value2"),
                        mapOf("key1" to "value1", "key2" to "value2")
                    )
                )
            )
        )

        val defaultList = ListTemplate(
            headerTitle = "WEEKLY MAGAZINE",
            headerLink = Link(
                webUrl = "https://developers.kakao.com",
                mobileWebUrl = "https://developers.kakao.com"
            ),
            contents = listOf(
                Content(
                    title = "취미의 특징, 탁구",
                    description = "스포츠",
                    imageUrl = "http://mud-kage.kakao.co.kr/dn/bDPMIb/btqgeoTRQvd/49BuF1gNo6UXkdbKecx600/kakaolink40_original.png",
                    link = Link(
                        webUrl = "https://developers.kakao.com",
                        mobileWebUrl = "https://developers.kakao.com"
                    )
                ),
                Content(
                    title = "크림으로 이해하는 커피이야기",
                    description = "음식",
                    imageUrl = "http://mud-kage.kakao.co.kr/dn/QPeNt/btqgeSfSsCR/0QJIRuWTtkg4cYc57n8H80/kakaolink40_original.png",
                    link = Link(
                        webUrl = "https://developers.kakao.com",
                        mobileWebUrl = "https://developers.kakao.com"
                    )
                ),
                Content(
                    title = "감성이 가득한 분위기",
                    description = "사진",
                    imageUrl = "http://mud-kage.kakao.co.kr/dn/c7MBX4/btqgeRgWhBy/ZMLnndJFAqyUAnqu4sQHS0/kakaolink40_original.png",
                    link = Link(
                        webUrl = "https://developers.kakao.com",
                        mobileWebUrl = "https://developers.kakao.com"
                    )
                )
            ),
            buttons = listOf(
                Button(
                    "웹으로 보기",
                    Link(
                        webUrl = "https://developers.kakao.com",
                        mobileWebUrl = "https://developers.kakao.com"
                    )
                ),
                Button(
                    "앱으로 보기",
                    Link(
                        androidExecParams = mapOf("key1" to "value1", "key2" to "value2"),
                        iosExecParams = mapOf("key1" to "value1", "key2" to "value2")
                    )
                )
            )
        )

        val defaultLocation = LocationTemplate(
            content = Content(
                title = "카카오톡 링크",
                description = "디폴트 템플릿 Location",
                imageUrl = "http://alpha-api1-kage.kakao.com/dn/cerDB5/ZSb2iRugKx/M4nuZxX823tnK1Mk5yVcv0/kakaolink40_original.png",
                link = Link(
                    webUrl = "https://developers.com",
                    mobileWebUrl = "https://developers.kakao.com"
                )
            ),
            social = Social(likeCount = 100, commentCount = 200),
            buttonTitle = "바로 확인",
            address = "경기 성남시 분당구 판교역로 235 에이치스퀘어 N동 7층",
            addressTitle = "카카오"
        )

        val defaultCommerce = CommerceTemplate(
            content = Content(
                title = "카카오톡 링크",
                description = "디폴트 템플릿 Commerce",
                imageUrl = "http://alpha-api1-kage.kakao.com/dn/cerDB5/ZSb2iRugKx/M4nuZxX823tnK1Mk5yVcv0/kakaolink40_original.png",
                link = Link(
                    webUrl = "https://developers.kakao.com",
                    mobileWebUrl = "https://developers.kakao.com"
                )
            ),
            commerce = Commerce(
                regularPrice = 10000,
                productName = "상품명"
            ),
            buttonTitle = "바로 확인"
        )

        val defaultText = TextTemplate(
            text = """
                카카오링크는 카카오 플랫폼 서비스의 대표 기능으로써 사용자의 모바일 기기에 설치된 카카오 플랫폼과 연동하여 다양한 기능을 실행할 수 있습니다.
                현재 이용할 수 있는 카카오링크는 다음과 같습니다.
                카카오톡링크
                카카오톡을 실행하여 사용자가 선택한 채팅방으로 메시지를 전송합니다.
                카카오스토리링크
                카카오스토리 글쓰기 화면으로 연결합니다.
            """.trimIndent(),
            link = Link(
                webUrl = "http://developers.kakao.com",
                mobileWebUrl = "http://dev.kakao.om"
            )
        )

    }
}
